# -*- coding: utf-8 -*-
#
# alone : web application framework for small embedded systems.
#               Copyright (c) 2009-2010 Inas Co Ltd. All Rights Reserved.
#
# This file is destributed under BSD License. Please read the COPYRIGHT file.
#
# apache httpd用 basic認証ファイル編集サンプル
#
#@note
# 使用前に、以下の項目を確認、編集してください。
#  パスワードファイルへのパス PASSWORD_FILE
#  htpasswd実行ファイルへのパス HTPASSWD
#  cgiの実行権限で、パスワードファイルが編集できるようパーミッションの調整

PASSWORD_FILE = "/tmp/passfile"
HTPASSWD = "/usr/sbin/htpasswd"

require 'al_form'
require 'al_template'


class AlController

  #================================================================
  # constructor
  #----------------------------------------------------------------
  #@note
  #================================================================
  def initialize()
    @form = AlForm.new
    @form.set_widgets([
      AlText.new( 'username', :label=>'ユーザ名', 
                  :required=>true, :validator=>/^[\w]+$/ ),
      AlPassword.new( 'password', :label=>'パスワード', 
                  :required=>true, :validator=>/^[\w]+$/ ),
      AlSubmit.new( 'submit1', :value=>'決定', 
                  :tag_attr=> {:style=>"float: right;"} )
    ])
    @form.action = Alone::make_uri( :action=>'commit' )
  end


  #================================================================
  # デフォルトアクション
  #----------------------------------------------------------------
  #@note
  # 登録ユーザ一覧を表示する
  # パスワードファイルがなければ、空ファイルを作成する
  #================================================================
  def action_index()
    @users = []
    begin
      File.open( PASSWORD_FILE, 'r' ) do |file|
        file.each do |text|
          @users << text.split(':')[0]
        end
      end
      
    rescue
      File.open( PASSWORD_FILE, 'w' ) {}

    end

    AlTemplate.run( 'v_index.rhtml' )
  end


  #================================================================
  # 新規登録アクション
  #----------------------------------------------------------------
  #@note
  # 空の登録フォームを表示
  #================================================================
  def action_createform()
    AlTemplate.run( 'v_form.rhtml' )
  end


  #================================================================
  # 更新アクション
  #----------------------------------------------------------------
  #@note
  # 更新用にフォームを調整してから表示
  #================================================================
  def action_updateform()
    if ! @form.fetch_request() || ! @form.validate([:username])
      return
    end
    @form.get_widget(:username).set_attr( :readonly=>'readonly' )
    AlTemplate.run( 'v_form.rhtml' )
  end


  #================================================================
  # 新規登録・更新の確定
  #----------------------------------------------------------------
  #@note
  # フォームからのPOSTを受け取って、登録・更新を行い結果を表示。
  # 利用しているコマンド htpasswd に登録と更新の区別がないので、
  # こちらもそのように実装。
  #================================================================
  def action_commit()
    if ! @form.fetch_request() || ! @form.validate()
      # バリデーションエラーならフォームへ戻す
      AlTemplate.run( 'v_form.rhtml' )
    else
      @result = `#{HTPASSWD} -b #{PASSWORD_FILE} #{@form[:username]} #{@form[:password]} 2>&1`
      @message = ($? == 0) ? '完了しました' : '失敗しました'
      AlTemplate.run( 'v_commit.rhtml' )
    end
  end


  #================================================================
  # 削除アクション
  #----------------------------------------------------------------
  #@note
  # 削除確認の表示
  #================================================================
  def action_deleteconfirm()
    if ! @form.fetch_request() || ! @form.validate([:username])
      return
    end
    AlTemplate.run( 'v_deleteconfirm.rhtml' )
  end


  #================================================================
  # 削除確定
  #----------------------------------------------------------------
  #@note
  # 削除を実行し、結果を表示
  #================================================================
  def action_deletecommit()
    if ! @form.fetch_request() || ! @form.validate([:username])
      return
    end
    @result = `#{HTPASSWD} -D #{PASSWORD_FILE} #{@form[:username]} 2>&1`
    @message = ($? == 0) ? '完了しました' : '失敗しました'
    AlTemplate.run( 'v_deletecommit.rhtml' )
  end

end
